@@ -4,6 +4,7 @@ import numpy as np
 
 from aif360.metrics import BinaryLabelDatasetMetric, utils
 from aif360.datasets import BinaryLabelDataset
+from aif360.datasets.multiclass_label_dataset import MulticlassLabelDataset
 
 
 class ClassificationMetric(BinaryLabelDatasetMetric):
@@ -32,20 +33,33 @@ class ClassificationMetric(BinaryLabelDatasetMetric):
             TypeError: `dataset` and `classified_dataset` must be
                 :obj:`~aif360.datasets.BinaryLabelDataset` types.
         """
-        if not isinstance(dataset, BinaryLabelDataset):
-            raise TypeError("'dataset' should be a BinaryLabelDataset")
+        if not isinstance(dataset, BinaryLabelDataset) and not isinstance(dataset, MulticlassLabelDataset) :
+            raise TypeError("'dataset' should be a BinaryLabelDataset or a MulticlassLabelDataset")
 
         # sets self.dataset, self.unprivileged_groups, self.privileged_groups
         super(ClassificationMetric, self).__init__(dataset,
             unprivileged_groups=unprivileged_groups,
             privileged_groups=privileged_groups)
 
-        if isinstance(classified_dataset, BinaryLabelDataset):
+        if isinstance(classified_dataset, BinaryLabelDataset) or isinstance(classified_dataset, MulticlassLabelDataset) :
             self.classified_dataset = classified_dataset
         else:
             raise TypeError("'classified_dataset' should be a "
-                            "BinaryLabelDataset.")
+                            "BinaryLabelDataset or a MulticlassLabelDataset.")
 
+        if isinstance(self.classified_dataset, MulticlassLabelDataset):
+            fav_label_value = 1.
+            unfav_label_value = 0.
+
+            self.classified_dataset = self.classified_dataset.copy()
+            # Find all the labels which match any of the favorable labels
+            fav_idx = np.logical_or.reduce(np.equal.outer(self.classified_dataset.favorable_label, self.classified_dataset.labels))
+            # Replace labels with corresponding values
+            self.classified_dataset.labels = np.where(fav_idx, fav_label_value, unfav_label_value)
+            
+            self.classified_dataset.favorable_label = float(fav_label_value)
+            self.classified_dataset.unfavorable_label = float(unfav_label_value)
+        
         # Verify if everything except the predictions and metadata are the same
         # for the two datasets
         with self.dataset.temporarily_ignore('labels', 'scores'):
@@ -182,7 +196,7 @@ class ClassificationMetric(BinaryLabelDatasetMetric):
 
     def num_generalized_false_positives(self, privileged=None):
         """Return the generalized number of false positives, :math:`GFP`, the
-        weighted sum of predicted scores where true labels are 'favorable',
+        weighted sum of predicted scores where true labels are 'unfavorable',
         optionally conditioned on protected attributes.
 
         Args:
@@ -200,7 +214,7 @@ class ClassificationMetric(BinaryLabelDatasetMetric):
 
     def num_generalized_false_negatives(self, privileged=None):
         """Return the generalized number of false negatives, :math:`GFN`, the
-        weighted sum of predicted scores where true labels are 'favorable',
+        weighted sum of 1 - predicted scores where true labels are 'favorable',
         optionally conditioned on protected attributes.
 
         Args:
@@ -218,7 +232,7 @@ class ClassificationMetric(BinaryLabelDatasetMetric):
 
     def num_generalized_true_negatives(self, privileged=None):
         """Return the generalized number of true negatives, :math:`GTN`, the
-        weighted sum of predicted scores where true labels are 'favorable',
+        weighted sum of 1 - predicted scores where true labels are 'unfavorable',
         optionally conditioned on protected attributes.
 
         Args:
@@ -763,10 +777,10 @@ class ClassificationMetric(BinaryLabelDatasetMetric):
         return self.generalized_entropy_index(alpha=1)
 
     def coefficient_of_variation(self):
-        r"""The coefficient of variation is two times the square root of the
+        r"""The coefficient of variation is the square root of two times the
         :meth:`generalized_entropy_index` with :math:`\alpha = 2`.
         """
-        return 2 * np.sqrt(self.generalized_entropy_index(alpha=2))
+        return np.sqrt(2*self.generalized_entropy_index(alpha=2))
 
     def between_group_theil_index(self):
         r"""The between-group Theil index is the
@@ -775,11 +789,11 @@ class ClassificationMetric(BinaryLabelDatasetMetric):
         return self.between_group_generalized_entropy_index(alpha=1)
 
     def between_group_coefficient_of_variation(self):
-        r"""The between-group coefficient of variation is two times the square
-        root of the :meth:`between_group_generalized_entropy_index` with
+        r"""The between-group coefficient of variation is the square
+        root of two times the :meth:`between_group_generalized_entropy_index` with
         :math:`\alpha = 2`.
         """
-        return 2*np.sqrt(self.between_group_generalized_entropy_index(alpha=2))
+        return np.sqrt(2*self.between_group_generalized_entropy_index(alpha=2))
 
     def between_all_groups_theil_index(self):
         r"""The between-group Theil index is the
@@ -789,13 +803,37 @@ class ClassificationMetric(BinaryLabelDatasetMetric):
         return self.between_all_groups_generalized_entropy_index(alpha=1)
 
     def between_all_groups_coefficient_of_variation(self):
-        r"""The between-group coefficient of variation is two times the square
-        root of the :meth:`between_all_groups_generalized_entropy_index` with
+        r"""The between-group coefficient of variation is the square
+        root of two times the :meth:`between_all_groups_generalized_entropy_index` with
         :math:`\alpha = 2`.
         """
-        return 2 * np.sqrt(self.between_all_groups_generalized_entropy_index(
+        return np.sqrt(2*self.between_all_groups_generalized_entropy_index(
             alpha=2))
 
+    def differential_fairness_bias_amplification(self, concentration=1.0):
+        """Bias amplification is the difference in smoothed EDF between the
+        classifier and the original dataset. Positive values mean the bias
+        increased due to the classifier.
+
+        Args:
+            concentration (float, optional): Concentration parameter for
+                Dirichlet smoothing. Must be non-negative.
+        """
+        ssr = self._smoothed_base_rates(self.classified_dataset.labels,
+                                        concentration)
+
+        def pos_ratio(i, j):
+            return abs(np.log(ssr[i]) - np.log(ssr[j]))
+
+        def neg_ratio(i, j):
+            return abs(np.log(1 - ssr[i]) - np.log(1 - ssr[j]))
+
+        edf_clf = max(max(pos_ratio(i, j), neg_ratio(i, j))
+                for i in range(len(ssr)) for j in range(len(ssr)) if i != j)
+        edf_data = self.smoothed_empirical_differential_fairness(concentration)
+
+        return edf_clf - edf_data
+
     # ============================== ALIASES ===================================
     def equal_opportunity_difference(self):
         """Alias of :meth:`true_positive_rate_difference`."""
